/*
 * Memory Setup stuff - taken from ???
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


#include <config.h>
#include <version.h>


/*
 * some parameters for the board
 */
SYSCON1:	.long	0x80000100
SYSCON2:	.long	0x80001100
SYSCON3:	.long	0x80002200
MEMCFG1:	.long	0x80000180
MEMCFG2:	.long	0x800001C0
SDCONF:		.long	0x80002300
SDRFPR:		.long	0x80002340

syscon1_val:	.long	0x00040100
syscon2_val:	.long	0x00000102
syscon3_val:	.long	0x0000020E
memcfg1_val:	.long	0x1f101710
memcfg2_mask:	.long	0x0000ffff	@ only set lower 16 bits
memcfg2_val:	.long	0x00001f13	@ upper 16 bits are reserved for CS7 + CS6
sdrfpr_val:	.long	0x00000240
sdconf_val:	.long	0x00000522

/*
 * setting up the memory :
 * 1. initialize SMC
 * 2. initialize DDRC
 */

.globl memsetup
memsetup:

	MOV	r11, lr

	/*
	 * set SMC config register as follows:
	 * bank enable, bank size = 8MByte, bank data width = 16Bit, disable write protect
	 */
	LDR	r0, =0x30000000
	LDR	r1, =0x10000031 
	STR	r1, [r0]   

	/*
	 * set DDRC config register as follows:
	 */

	/* Disable DDRC bank, and configure DDRC's bank configuration */
	LDR	r3, =0x4000000C


#if (DDRC_DATA_BUS_WIDTH == DDRC_DATA_BUS_WIDTH_32)


#if (DDRC_SDRAM_MODULE_SIZE == DDRC_SDRAM_SIZE_512MBIT)
	LDR	r4, =0x000000B1		  /* DDRC: 32-bit, DDR SDRAM: 512Mb, x16 device  */

#elif (DDRC_SDRAM_MODULE_SIZE == DDRC_SDRAM_SIZE_256MBIT)

#if 1
	LDR	r4, =0x000000A1		  /* DDRC: 32-bit, DDR SDRAM: 256Mb, x16 device  */
#else	/* for testing only */
	LDR	r4, =0x00000061		  /* DDRC: 32-bit, DDR SDRAM: 256Mb, x8 device   */
#endif

#elif (DDRC_SDRAM_MODULE_SIZE == DDRC_SDRAM_SIZE_128MBIT)
	LDR	r4, =0x00000091		  /* DDRC: 32-bit, DDR SDRAM: 128Mb, x16 device  */
	
#else
	LDR	r4, =0x00000081		  /* DDRC: 32-bit, DDR SDRAM: 64Mb, x16 device  */
	
#endif


#else	/* for #if (DDRC_DATA_BUS_WIDTH == DDRC_DATA_BUS_WIDTH_16) */


#if (DDRC_SDRAM_MODULE_SIZE == DDRC_SDRAM_SIZE_512MBIT)
	LDR	r4, =0x000000B0		  /* DDRC: 16-bit, DDR SDRAM: 512Mb, x16 device  */
	
#elif (DDRC_SDRAM_MODULE_SIZE == DDRC_SDRAM_SIZE_256MBIT)

#if 1
	LDR	r4, =0x000000A0		  /* DDRC: 16-bit, DDR SDRAM: 256Mb, x16 device  */
#else	/* for testing only */
	LDR	r4, =0x00000060		  /* DDRC: 16-bit, DDR SDRAM: 256Mb, x8 device   */
#endif

#elif (DDRC_SDRAM_MODULE_SIZE == DDRC_SDRAM_SIZE_128MBIT)
	LDR	r4, =0x00000090		  /* DDRC: 16-bit, DDR SDRAM: 128Mb, x16 device  */
	
#else
	LDR	r4, =0x00000080		  /* DDRC: 16-bit, DDR SDRAM: 64Mb, x16 device  */
	
#endif


#endif	/* end of #if (DDRC_DATA_BUS_WIDTH == DDRC_DATA_BUS_WIDTH_32) */

	/*LDR	r4, =0x000000A0*/	/* alan */
	STR	r4, [r3]


       
	/* configure DDRC timing parameter 0 register */
	LDR	r3, =0x40000000
	LDR	r4, =0x009EEA88
	STR	r4, [r3]
        

	/* configure DDRC timing parameter 1 register */
	LDR	r3, =0x40000004

#if (DDRC_SDRAM_MODULE_SIZE == DDRC_SDRAM_SIZE_512MBIT)
	LDR	r4, =0x000081E2
	
#elif (DDRC_SDRAM_MODULE_SIZE == DDRC_SDRAM_SIZE_256MBIT)
	LDR	r4, =0x000083C5
	
#elif (DDRC_SDRAM_MODULE_SIZE == DDRC_SDRAM_SIZE_128MBIT)
	LDR	r4, =0x0000878A
	
#elif (DDRC_SDRAM_MODULE_SIZE == DDRC_SDRAM_SIZE_64MBIT)
	LDR	r4, =0x00008F14
	
#endif
	/*LDR	r4, =0x000083C5  alan */
	STR	r4, [r3]
        
        
	/* configure DDRC configuration register;Init Seq */
	LDR	r3, =0x40000008
	LDR	r4, =0x00000001
	STR	r4, [r3]


	/* Waiting for Init done */        
wait:     
	LDR	r3, =0x40000008
	LDR	r4, [r3]
	BIC	r4, r4, #0xfffffffd
	CMP	r4, #0x00000002	/* icmp bit */
	BNE	wait


	/* configure DDRC DQs output delay */
	LDR	r3, =0x40000014
	LDR	r4, =0x00074444
	STR	r4, [r3]

   
	/* enable DDRC bank configuration */
	LDR	r3, =0x4000000C

	LDR	r4, [r3]

	ORR	r4, r4, #0x00000100

	STR	r4, [r3]


	MOV	pc, r11          	   

