#include "../../str9100_cfg.h"

#include "str9100/porting.h"
#include "str9100/star_flash.h"
#include "str9100/star_smc.h"


void       Flash_Reset(void);
void       Flash_Read_Silicon_ID(u_int32 *, u_int32 *);
u_int32    Flash_Chip_Erase(void);
u_int32    Flash_Sectors_Erase(u_int32, u_int32);
u_int32    Flash_Word_Write(u_int32, u_int16);
void       Flash_Word_Read(u_int32, u_int16 *);
u_int32    Check_Toggle_Bit_Ready(u_int32);
u_int32    Erase_Polling_Check(u_int32);
u_int32    Data_Polling_Check_Word(u_int32, u_int16);



/*
 * Note here the address value means system memory address
 */
u_int32 SA_Base_Addr_Table[MAX_SECTOR_INDEX + 1] = 
{ 
  SA0_BASE_ADDR,   SA1_BASE_ADDR,   SA2_BASE_ADDR,   SA3_BASE_ADDR,   SA4_BASE_ADDR, 
  SA5_BASE_ADDR,   SA6_BASE_ADDR,   SA7_BASE_ADDR,   SA8_BASE_ADDR,   SA9_BASE_ADDR, 
  SA10_BASE_ADDR,  SA11_BASE_ADDR,  SA12_BASE_ADDR,  SA13_BASE_ADDR,  SA14_BASE_ADDR,
  SA15_BASE_ADDR,  SA16_BASE_ADDR,  SA17_BASE_ADDR,  SA18_BASE_ADDR,  SA19_BASE_ADDR,
  SA20_BASE_ADDR,  SA21_BASE_ADDR,  SA22_BASE_ADDR,  SA23_BASE_ADDR,  SA24_BASE_ADDR,
  SA25_BASE_ADDR,  SA26_BASE_ADDR,  SA27_BASE_ADDR,  SA28_BASE_ADDR,  SA29_BASE_ADDR,
  SA30_BASE_ADDR,  SA31_BASE_ADDR,  SA32_BASE_ADDR,  SA33_BASE_ADDR,  SA34_BASE_ADDR,
  SA35_BASE_ADDR,  SA36_BASE_ADDR,  SA37_BASE_ADDR,  SA38_BASE_ADDR,  SA39_BASE_ADDR,
  SA40_BASE_ADDR,  SA41_BASE_ADDR,  SA42_BASE_ADDR,  SA43_BASE_ADDR,  SA44_BASE_ADDR,
  SA45_BASE_ADDR,  SA46_BASE_ADDR,  SA47_BASE_ADDR,  SA48_BASE_ADDR,  SA49_BASE_ADDR,
  SA50_BASE_ADDR,  SA51_BASE_ADDR,  SA52_BASE_ADDR,  SA53_BASE_ADDR,  SA54_BASE_ADDR,
  SA55_BASE_ADDR,  SA56_BASE_ADDR,  SA57_BASE_ADDR,  SA58_BASE_ADDR,  SA59_BASE_ADDR,
  SA60_BASE_ADDR,  SA61_BASE_ADDR,  SA62_BASE_ADDR,  SA63_BASE_ADDR,  SA64_BASE_ADDR,
  SA65_BASE_ADDR,  SA66_BASE_ADDR,  SA67_BASE_ADDR,  SA68_BASE_ADDR,  SA69_BASE_ADDR,
  SA70_BASE_ADDR,  SA71_BASE_ADDR,  SA72_BASE_ADDR,  SA73_BASE_ADDR,  SA74_BASE_ADDR,
  SA75_BASE_ADDR,  SA76_BASE_ADDR,  SA77_BASE_ADDR,  SA78_BASE_ADDR,  SA79_BASE_ADDR,
  SA80_BASE_ADDR,  SA81_BASE_ADDR,  SA82_BASE_ADDR,  SA83_BASE_ADDR,  SA84_BASE_ADDR,
  SA85_BASE_ADDR,  SA86_BASE_ADDR,  SA87_BASE_ADDR,  SA88_BASE_ADDR,  SA89_BASE_ADDR,
  SA90_BASE_ADDR,  SA91_BASE_ADDR,  SA92_BASE_ADDR,  SA93_BASE_ADDR,  SA94_BASE_ADDR,
  SA95_BASE_ADDR,  SA96_BASE_ADDR,  SA97_BASE_ADDR,  SA98_BASE_ADDR,  SA99_BASE_ADDR,
  SA100_BASE_ADDR, SA101_BASE_ADDR, SA102_BASE_ADDR, SA103_BASE_ADDR, SA104_BASE_ADDR,
  SA105_BASE_ADDR, SA106_BASE_ADDR, SA107_BASE_ADDR, SA108_BASE_ADDR, SA109_BASE_ADDR,
  SA110_BASE_ADDR, SA111_BASE_ADDR, SA112_BASE_ADDR, SA113_BASE_ADDR, SA114_BASE_ADDR,
  SA115_BASE_ADDR, SA116_BASE_ADDR, SA117_BASE_ADDR, SA118_BASE_ADDR, SA119_BASE_ADDR,
  SA120_BASE_ADDR, SA121_BASE_ADDR, SA122_BASE_ADDR, SA123_BASE_ADDR, SA124_BASE_ADDR,
  SA125_BASE_ADDR, SA126_BASE_ADDR, SA127_BASE_ADDR, SA128_BASE_ADDR, SA129_BASE_ADDR,
  SA130_BASE_ADDR, SA131_BASE_ADDR, SA132_BASE_ADDR, SA133_BASE_ADDR, SA134_BASE_ADDR
};




/*
 * Function : Flash_Reset
 * Purpose : This function is to software reset the flash chips
 */
void Flash_Reset(void)
{
    int volatile i;


    *(u_int16 volatile *)(SA0_BASE_ADDR) = 0xF0;
    
    // delay some time
    for (i = DELAY_LOOP; i > 0; i--);
}


/*
 * Function : Flash_Read_Silicon_ID
 * Purpose : This function is to read manufacture ID and device ID of flash
 */
void Flash_Read_Silicon_ID(u_int32 *manufacture_ID, u_int32 *device_ID)
{
    // reset the flash silicon
    Flash_Reset();
	
    /*
     * get flash silicon manufacture ID
     */
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0xAA;
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x2AA << BIT_SHIFT_NUM)) = 0x55;
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0x90;
    
    Check_Toggle_Bit_Ready(MANUFACTURE_ID_ADDR);
        
    *manufacture_ID = (u_int32)(*(u_int16 volatile *)(MANUFACTURE_ID_ADDR));


    // reset the flash silicon again
    Flash_Reset();  
    
    /*
     * get flash silicon device ID
     */
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0xAA;
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x2AA << BIT_SHIFT_NUM)) = 0x55;
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0x90;
    
    Check_Toggle_Bit_Ready(DEVICE_ID_ADDR);
    
    *device_ID = (u_int32)(*(u_int16 volatile *)(DEVICE_ID_ADDR));

    // reset the flash silicon again
    Flash_Reset();
}


/*
 * Function : Flash_Chip_Erase
 * Purpose : This function is to erase the whole flash chip
 */
u_int32 Flash_Chip_Erase(void)
{    
    u_int32  volatile i;
    
    /*
     * automatic chip erase algorithm
     */
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0xAA;
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x2AA << BIT_SHIFT_NUM)) = 0x55;
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0x80;
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0xAA;
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x2AA << BIT_SHIFT_NUM)) = 0x55;
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0x10;

    
    // data polling to verify the chip erase operation is finished
    for (i = 0; i <= MAX_SECTOR_INDEX; i++)
    {
        if (!Erase_Polling_Check(SA_Base_Addr_Table[i]))   
        {
            printf("\nFail to erase flash chip!\n");
    	
            // reset the flash silicon
            Flash_Reset();
        
            return FALSE;           
        }           
    }  // end of for (i = 0; i <= MAX_SECTOR_INDEX; i++)
    
    printf("\nFlash chip is erased successfully.\n");
        
    return TRUE;
}



/*
 * Function : Flash_Sectors_Erase
 * Purpose : This function is to erase sectors of flash
 */
u_int32 Flash_Sectors_Erase(u_int32 start_sector_num, u_int32 end_sector_num)
{
    int        i;
    u_int32    status = TRUE;


    // reset the flash silicon
    Flash_Reset();

    if (start_sector_num == 0 && end_sector_num == MAX_SECTOR_INDEX)
    {
    	// chip erase	    	
        return (Flash_Chip_Erase());
    }           
    
    for (i = start_sector_num; i <= end_sector_num; ++i)
    {
    	/*
         * automatic sector erase algorithm
         */
        *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0xAA;
        *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x2AA << BIT_SHIFT_NUM)) = 0x55;
        *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0x80;
        *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0xAA;
        *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x2AA << BIT_SHIFT_NUM)) = 0x55;
    	
        *(u_int16 volatile *)SA_Base_Addr_Table[i] = 0x30;
        

        // data polling to verify the erase operation is finished
        if (Erase_Polling_Check(SA_Base_Addr_Table[i]))
        {
            printf("\nFlash sector %d is erased successfully.\n", i);
        }
        else
        {
            printf("\nError : fail to erase flash sector %d!\n", i);
            
            // reset the flash silicon
            Flash_Reset();
            
            status = FALSE;
            
            break;
        }
    }  // end of for (i = start_sector_num; i <= end_sector_num; ++i)
    
    return status;
}


/*
 * Function : Flash_Word_Write
 * Purpose : This function is to write word data(2 bytes) into flash
 */
u_int32 Flash_Word_Write(u_int32 write_address, u_int16 write_data)
{    
    /*
     * automatic programming(writing) algorithm
     */
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0xAA;
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x2AA << BIT_SHIFT_NUM)) = 0x55;
    *(u_int16 volatile *)(SYSTEM_FLASH_BASE_ADDR + (0x555 << BIT_SHIFT_NUM)) = 0xA0;
    
    *(u_int16 volatile *)(write_address) = write_data;
    
    return (Data_Polling_Check_Word(write_address, write_data));
}


/*
 * Function : Flash_Word_Read
 * Purpose : This function is to read word data(2 bytes) from flash
 */
void Flash_Word_Read(u_int32 read_address, u_int16 *read_data)
{    
    *read_data = *(u_int16 volatile *)read_address;
}


/*
 * Function : Check_Toggle_Bit_Ready
 * Purpose : This function is to check the toggle bit Q6 to check if program/erase operation is 
 *           complete
 */
u_int32 Check_Toggle_Bit_Ready(u_int32 read_address)
{
    int               i;
    u_int32           status = FALSE;
    u_int16 volatile  prev_read_data_word = 0;
    u_int16 volatile  curr_read_data_word = 0;
    

    // read data
    prev_read_data_word = *(u_int16 volatile *)read_address;
    
    for (i = 0x07FFFFFF; i > 0; i--)        
    {        
    	// read data again
        curr_read_data_word = *(u_int16 volatile *)read_address;
        
        if ((prev_read_data_word & TOGGLE_BIT_MASK) == (curr_read_data_word & TOGGLE_BIT_MASK))
        {
            // stop toggling, program/erase operation is complete
            return TRUE;
        }        
       
        // check Q5 bit for time-out
        if ((curr_read_data_word & TIME_OUT_MASK) == TIME_OUT_MASK)  // time-out
        {
            prev_read_data_word = *(u_int16 volatile *)read_address;
            
            curr_read_data_word = *(u_int16 volatile *)read_address;
            
            if ((prev_read_data_word & TOGGLE_BIT_MASK) == (curr_read_data_word & TOGGLE_BIT_MASK))
            {
            	// stop toggling, program/erase operation is complete
                return TRUE;                
            }  
            else
            {
            	// toggling is continuing , so issue reset command
            	Flash_Reset();
            	
            	break;
            }
        }   
        else  // not time-out
        {
            prev_read_data_word = curr_read_data_word;
        }            
    }  // end of for (i = 0x07FFFFFF; i > 0; i--)
    
    return status;
}


/*
 * Function : Erase_Polling_Check
 * Purpose : This function is to check the erase operation is complete or not
 */
u_int32 Erase_Polling_Check(u_int32 erase_sector_addr)
{
    int               i;
    u_int32           status = FALSE;
    u_int16 volatile  poll_data_word = 0;    
        
    
    for (i = 0x07FFFFFF; i > 0; i--)
    {
    	// read data
        poll_data_word = *(u_int16 volatile *)erase_sector_addr;       
        
        if (poll_data_word == 0xFFFF)  // erase is complete
        {
            return TRUE;
        }        
        
        if ((poll_data_word & TIME_OUT_MASK) == TIME_OUT_MASK)  // time-out
        { 
            // read data again
            poll_data_word = *(u_int16 volatile *)erase_sector_addr;          
            
            return (poll_data_word == 0xFFFF) ? TRUE : FALSE;
        }       
    }
    
    return status;
}


/*
 * Function : Data_Polling_Check_Word
 * Purpose : This function is to check the writing operation is complete or not
 */
u_int32 Data_Polling_Check_Word(u_int32 write_address, u_int16 write_data_word)
{
    u_int32             i;
    u_int16 volatile    read_data_word = 0;   
    
    for (i = 0x0005FFFF; i > 0; --i)
    {    	
    	read_data_word = *(u_int16 volatile *)write_address; 
    	
    	// for testing only
    	if (read_data_word == write_data_word)
    	{
    	    //printf("[%d]rdata=0x%04x, wdata=0x%04x\n", i, read_data_word, write_data_word);
    	    break;
    	}
#if 0        
        if ((read_data_word & DATA_POLL_MASK) == (write_data_word & DATA_POLL_MASK))
        {
            break;  // writing is complete
        }
        
        if ((read_data_word & TIME_OUT_MASK) == TIME_OUT_MASK)
        {
            // writing is time-out, read data again
            read_data_word = *(u_int16 volatile *)write_address;
            
            if ((read_data_word & DATA_POLL_MASK) == (write_data_word & DATA_POLL_MASK))
            {
                return TRUE;
            }
            else
            {
                return FALSE;
            }
        }             
#endif
    }  // end of for (i = 0x37FFFFFF; i > 0; --i)
    	
    return (i > 0) ? TRUE : FALSE;    
}