/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/arch/hardware.h> 


/*
 * Miscelaneous platform dependent initialisations
 */
int board_init(void)
{
	DECLARE_GLOBAL_DATA_PTR;

	/* Enable Ctrlc */
	console_init_f ();
    /* 
     * architecture number MACH_TYPE_STR9100
     */
	
    gd->bd->bi_arch_number = 242;

    
    /* adress of boot parameters */
    gd->bd->bi_boot_params = PHYS_SDRAM + 0x100;

    matrix400_gpio_init();	

    return 0;
}


void sdram_config(int bank_num, u32 bank_base, int data_width, int sdram_size, int bus_width, int bank_size)
{
    u32    config_reg_val;


    config_reg_val = *(volatile u32 *)(0x40000000 + 0x0C);

    config_reg_val &= ~(0xFF);
    
    config_reg_val |= ((data_width & 0x3) << 6) | ((sdram_size & 0x3) << 4) | (bus_width & 0x1);
    
    *(volatile u32 *)(0x40000000 + 0x0C) = config_reg_val;
}



int dram_init(void)
{
	DECLARE_GLOBAL_DATA_PTR;
    int    i;
    u32    config_reg_val;
    u32    phys_sdram_size = 0;
    u32    phys_sdram_data_bus = 0;
    


    /*
     * Only one DDR SDRAM bank
     */
    for (i = 0; i < 1; ++i)
    {        
        config_reg_val = *(volatile u32 *)(0x40000000 + 0x0C);
		
        if ((config_reg_val & (0x1 << 8)) == 0)
	{
	    continue;
	}
		
	switch ((config_reg_val >> 4) & 0x3)
	{
	    case 0:
	        phys_sdram_size += 0x800000;	/* 8 MByte */
	        break;
	        
	    case 1:
	        phys_sdram_size += 0x1000000;	/* 16 MByte */
	        break;
	        
	    case 2:
	        phys_sdram_size += 0x2000000;	/* 32 MByte */
	        break;
	        
    	    case 3:
    	        phys_sdram_size += 0x4000000;	/* 64 MByte */
    	        break;
        }
     

        phys_sdram_data_bus = (config_reg_val >> 6) & 0x3;


        if (config_reg_val & 0x1)
        {
            /*
             * DDRC : 32-bit memory bus width
             */
            if (phys_sdram_data_bus == 0)
            {
                 /* DDR SDRAM x4 device */
                 phys_sdram_size = phys_sdram_size * (32 / 4);
            }
            else if (phys_sdram_data_bus == 1)
            {
                 /* DDR SDRAM x8 device */
                 phys_sdram_size = phys_sdram_size * (32 / 8);
            }
            else if (phys_sdram_data_bus == 2)
            {
                 /* DDR SDRAM x16 device */
                 phys_sdram_size = phys_sdram_size * (32 / 16);
            }
            else
            {
                printf("\nError : incorrect DDR SDRAM Data Width Setting in DDRC!\n");
            }
        }
        else
        {
            /*
             * DDRC : 16-bit memory bus width
             */
            if (phys_sdram_data_bus == 0)
            {
                 /* DDR SDRAM x4 device */
                 phys_sdram_size = phys_sdram_size * (16 / 4);
            }
            else if (phys_sdram_data_bus == 1)
            {
                 /* DDR SDRAM x8 device */
                 phys_sdram_size = phys_sdram_size * (16 / 8);
            }
            else if (phys_sdram_data_bus == 2)
            {
                 /* DDR SDRAM x16 device */
                 phys_sdram_size = phys_sdram_size * (16 / 16);
            }
            else
            {
                printf("\nError : incorrect DDR SDRAM Data Width Setting in DDRC!\n");
            }
        }

    }

	gd->bd->bi_dram[0].start = PHYS_SDRAM;
	
        gd->bd->bi_dram[0].size  = phys_sdram_size;

	for(i=0;i<500;i++){
	HAL_GPIO_SET_DATA_OUT_LOW(MATRIX400_READY);
	HAL_GPIO_SET_DATA_OUT_LOW(MATRIX400_BUZZER);
	udelay(350*4);
	HAL_GPIO_SET_DATA_OUT_HIGH(MATRIX400_READY);
	HAL_GPIO_SET_DATA_OUT_HIGH(MATRIX400_BUZZER);
	udelay(180*4);
	
	}
	HAL_GPIO_SET_DATA_OUT_LOW(MATRIX400_READY);
    return 0;//phys_sdram_size;
}
